import 'package:nginx_le_shared/src/util/env_var.dart';

import '../../nginx_le_shared.dart';
import '../config/ConfigYaml.dart';

abstract class AuthProvider {
  String get name;

  String get summary;

  /// Provides a list of environment variables that must be passed into
  /// the docker container when it is created.
  List<EnvVar> get environment;

  void promptForSettings(ConfigYaml confi);

  /// Starts the process to acquire a certificate.
  void acquire();

  /// overload this method if your provide needs to to have a manual auth_hook called
  void auth_hook();

  /// overload this method if your provide needs to to have a manual cleanup hook called
  void cleanup_hook();

  /// Overload this method to indicate whether the auth provider can support
  /// authentication of wildcard certificates.
  bool get supportsWildCards;

  /// Overload this method to indicate if the auth provider supports a webserver
  /// operating on a private ip address (with no public access).
  bool get supportsPrivateMode;
}
